package com.ym.mec.biz.dal.dto;


import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;

/**
 * 直播房间管理表(ImLiveBroadcastRoom)表实体类
 *
 * @author hgw
 * @since 2022-02-17 20:52:05
 */
@ApiModel(value = "直播房间管理")
public class ImLiveBroadcastRoomDto implements Serializable {
    @ApiModelProperty(value = "主键")
    private Integer id;

    @ApiModelProperty(value = "机构id")
    private Integer tenantId;

    @NotNull(message = "主讲人不能为空!")
    @ApiModelProperty(value = "主讲人id/老师id")
    private Integer speakerId;

    @Size(max = 13, message = "房间标题最多12个字!")
    @NotBlank(message = "房间标题不能为空")
    @ApiModelProperty(value = "房间标题/最多12个字")
    private String roomTitle;

    @NotNull(message = "直播开始时间不能为空!")
    @ApiModelProperty(value = "直播开始时间")
    private Date liveStartTime;

    @Size(max = 13, message = "直播内容最多200个字!")
    @NotBlank(message = "直播内容不能为空")
    @ApiModelProperty(value = "直播内容/最多200个字")
    private String liveRemark;

    @NotBlank(message = "预热模版不能为空")
    @ApiModelProperty(value = "预热模版")
    private String preTemplate;

    @ApiModelProperty(value = "房间配置")
    private RoomConfig roomConfig;

    @ApiModel(value = "房间配置")
    public static class RoomConfig implements Serializable {

        @ApiModelProperty(value = "是否允许点赞 0允许 1不允许")
        private Integer whether_like = 0;

        @ApiModelProperty(value = "是否允许聊天互动  0允许 1不允许")
        private Integer whether_chat = 0;

        @ApiModelProperty(value = "是否允许保存直播回放 0允许 1不允许")
        private Integer whether_video = 1;

        public Integer getWhether_like() {
            return whether_like;
        }

        public void setWhether_like(Integer whether_like) {
            this.whether_like = whether_like;
        }

        public Integer getWhether_chat() {
            return whether_chat;
        }

        public void setWhether_chat(Integer whether_chat) {
            this.whether_chat = whether_chat;
        }

        public Integer getWhether_video() {
            return whether_video;
        }

        public void setWhether_video(Integer whether_video) {
            this.whether_video = whether_video;
        }
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Integer getTenantId() {
        return tenantId;
    }

    public void setTenantId(Integer tenantId) {
        this.tenantId = tenantId;
    }

    public Integer getSpeakerId() {
        return speakerId;
    }

    public void setSpeakerId(Integer speakerId) {
        this.speakerId = speakerId;
    }

    public String getRoomTitle() {
        return roomTitle;
    }

    public void setRoomTitle(String roomTitle) {
        this.roomTitle = roomTitle;
    }

    public Date getLiveStartTime() {
        return liveStartTime;
    }

    public void setLiveStartTime(Date liveStartTime) {
        this.liveStartTime = liveStartTime;
    }

    public String getLiveRemark() {
        return liveRemark;
    }

    public void setLiveRemark(String liveRemark) {
        this.liveRemark = liveRemark;
    }

    public String getPreTemplate() {
        return preTemplate;
    }

    public void setPreTemplate(String preTemplate) {
        this.preTemplate = preTemplate;
    }

    public RoomConfig getRoomConfig() {
        return roomConfig;
    }

    public void setRoomConfig(RoomConfig roomConfig) {
        this.roomConfig = roomConfig;
    }

}

