package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.CloudTeacherOrderDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.page.CloudTeacherAddQueryInfo;
import com.ym.mec.biz.dal.page.CloudTeacherOrderQueryInfo;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;


public interface CloudTeacherOrderService extends BaseService<Long, CloudTeacherOrder> {

    /**
     * 查询未激活用户
     */
    PageInfo<CloudTeacherStudent> queryInactive(CloudTeacherOrderQueryInfo cloudTeacherOrderQueryInfo) throws Exception;


    /**
     * 将订单详情云教练加入云教练购买列表
     *
     * @param order
     * @return
     */
    void addOrderDetail2CloudTeacher(StudentPaymentOrder order, BigDecimal cloudTeacherFee);


    /**
     * 将生效的云教练订单加入
     *
     * @param cloudTeacherOrder
     * @return
     */
    CloudTeacher addStudentCloudTeacher(CloudTeacherOrder cloudTeacherOrder);

    Date calcCloudTeacherOrderTime(CloudTeacherOrder cloudTeacherOrder, Date startTime, Date endTime);

    /**
     * 获取乐团云教练订单
     *
     * @param studentIds
     * @param musicGroupId
     * @return
     */
    List<CloudTeacherOrder> getStudentCloudTeacherOrders(List<Integer> studentIds, String musicGroupId);

    /**
     * 获取乐团报名的云教练订单
     *
     * @param studentId
     * @return
     */
    List<CloudTeacherOrder> getStudentCloudTeacherOrder(Integer studentId, String musicGroupId);

    /**
     * 退云教练
     *
     * @param cloudTeacherOrders
     * @param refundAmount
     * @return
     */
    Boolean quitCloudTeacherOrders(List<CloudTeacherOrder> cloudTeacherOrders, BigDecimal refundAmount);

    /**
     * 退云教练
     *
     * @param order
     * @param refundAmount
     * @return
     */
    Boolean quitCloudTeacherOrder(CloudTeacherOrder order, BigDecimal refundAmount);

    /**
     * 根据studentPaymentOrder表id查询对象
     *
     * @param orderId
     * @return
     */
    CloudTeacherOrder queryByOrderId(Long orderId);

    /**
     * 根据订单编号查询订单详情
     *
     * @param orderId
     * @return
     */
    CloudTeacherOrderDto queryOrderInfoByOrderId(Long orderId);

    List<Map<Long, String>> queryNoStartByUserIds(Set<Integer> userIds, Boolean isMusicMember);

    /**
     * 获取学员购买的云教练活动订单
     *
     * @param userId
     * @param activeRemark
     * @param remark
     * @return
     */
    String queryActiveOrderPage(Integer userId, Integer activeRemark, String remark);

    /**
     * 支付未激活用户
     *
     * @param cloudTeacherStudents 选择中用户清单
     * @return 是否成功
     */
    Map<String, Object> pay(List<CloudTeacherStudent> cloudTeacherStudents) throws Exception;

    Boolean payCheck(String orderNo);

    PageInfo<CloudTeacherStudent> queryRecord(CloudTeacherOrderQueryInfo cloudTeacherOrderQueryInfo);

    boolean addStudents(CloudTeacherAddQueryInfo cloudTeacherAddQueryInfo);

    int sendSms(TenantOrderRecord record);
}
