package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.CloudCoachPaymentProgram;
import com.ym.mec.biz.dal.entity.CloudTeacherOrder;
import com.ym.mec.biz.dal.entity.MemberRankSetting;
import com.ym.mec.biz.dal.entity.TenantInfo;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.service.CloudCoachPaymentProgramService;
import com.ym.mec.biz.service.MemberFeeSettingService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.http.HttpUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class CloudCoachPaymentProgramServiceImpl extends BaseServiceImpl<Long, CloudCoachPaymentProgram>  implements CloudCoachPaymentProgramService {
	
	@Autowired
	private CloudCoachPaymentProgramDao cloudCoachPaymentProgramDao;
	@Autowired
	private StudentCloudCoachPaymentDetailsDao studentCloudCoachPaymentDetailsDao;
	@Autowired
	private MemberFeeSettingService memberFeeSettingService;
	@Autowired
	private MemberRankSettingDao memberRankSettingDao;
	@Autowired
	private CloudTeacherOrderDao cloudTeacherOrderDao;
	@Autowired
	private TenantInfoDao tenantInfoDao;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private TeacherDao teacherDao;

	@Override
	public BaseDAO<Long, CloudCoachPaymentProgram> getDAO() {
		return cloudCoachPaymentProgramDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void add(CloudCoachPaymentProgram cloudCoachPaymentProgram) {
		List<Integer> userIds = cloudCoachPaymentProgram.getUserIds();
		if(userIds == null || userIds.size() == 0){
			throw new BizException("请选择学员");
		}
		Integer tenantId = TenantContextHolder.getTenantId();
		//获取原价
		BigDecimal originalFee = memberFeeSettingService.getAmount(cloudCoachPaymentProgram.getOrganId(),
				cloudCoachPaymentProgram.getMemberRankSettingId(),
				cloudCoachPaymentProgram.getPeriod(),
				cloudCoachPaymentProgram.getMemberNum(),
				MemberFeeSettingServiceImpl::getOriginalFee);
		cloudCoachPaymentProgram.setOriginalAmount(originalFee);
		//获取会员名称
		MemberRankSetting memberRankSetting = memberRankSettingDao.get(cloudCoachPaymentProgram.getMemberRankSettingId());
		cloudCoachPaymentProgram.setName(memberRankSetting.getName());
		cloudCoachPaymentProgram.setUserNum(userIds.size());
		cloudCoachPaymentProgram.setTenantId(tenantId);
		//是否0元购买
		Boolean isFree = cloudCoachPaymentProgram.getActualAmount().compareTo(BigDecimal.ZERO) == 0;
		if(isFree){cloudCoachPaymentProgram.setPaymentUserNum(userIds.size());}
		cloudCoachPaymentProgramDao.insert(cloudCoachPaymentProgram);

		//批量插入学员缴费项目
		studentCloudCoachPaymentDetailsDao.batchInsert(cloudCoachPaymentProgram.getId(),isFree?1:0,userIds,tenantId);
		//如果是0元订单，保存为待激活的云教练订单
		if(isFree){
			CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
			cloudTeacherOrder.setTime(cloudCoachPaymentProgram.getMemberNum().intValue());
			cloudTeacherOrder.setOrganId(cloudCoachPaymentProgram.getOrganId());
			cloudTeacherOrder.setType(cloudCoachPaymentProgram.getPeriod());
			cloudTeacherOrder.setLevel(cloudCoachPaymentProgram.getMemberRankSettingId());
			cloudTeacherOrder.setAmount(BigDecimal.ZERO);
			cloudTeacherOrder.setStatus(1);
			cloudTeacherOrder.setRemark(cloudCoachPaymentProgram.getRemark());
			cloudTeacherOrderDao.batchInsert(cloudTeacherOrder,cloudCoachPaymentProgram.getUserIds());
		}else {
			//推送缴费消息
			TenantInfo tenantInfo = tenantInfoDao.selectById(cloudCoachPaymentProgram.getTenantId());
			Map<Integer, String> phoneMaps = MapUtil.convertMybatisMap(teacherDao.queryPhoneByIds(StringUtils.join(userIds,",")));
			String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
			StringBuffer pushUrl = new StringBuffer(baseApiUrl).append("/#/studentMember?id=").append(cloudCoachPaymentProgram.getId());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.STUDENT_SMS_CLOUD_PAYMENT,phoneMaps,
					null,0,"",null,tenantInfo.getName(), HttpUtil.getSortUrl(pushUrl.toString()));

			StringBuffer notifyUrl = new StringBuffer("8?").append(baseApiUrl).append("/#/studentMember?id=").append(cloudCoachPaymentProgram.getId());
			Map<Integer, String> userIdMaps = new HashMap<>(userIds.size());
			userIds.forEach(e->userIdMaps.put(e,e.toString()));
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_CLOUD_PAYMENT,userIdMaps,
					null,0,notifyUrl.toString(),"STUDENT",tenantInfo.getName());
		}
	}
}