package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ym.mec.biz.dal.dao.CloudTeacherDao;
import com.ym.mec.biz.dal.dao.CloudTeacherOrderDao;
import com.ym.mec.biz.dal.dto.CloudTeacherOrderDto;
import com.ym.mec.biz.dal.dto.Mapper;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.PaymentChannelEnum;
import com.ym.mec.biz.dal.enums.TenantOrderRecordEnum;
import com.ym.mec.biz.dal.page.CloudTeacherAddQueryInfo;
import com.ym.mec.biz.dal.page.CloudTeacherOrderQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.WrapperUtil;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import jodd.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.PeriodEnum.*;

@Service
public class CloudTeacherOrderServiceImpl extends BaseServiceImpl<Long, CloudTeacherOrder> implements CloudTeacherOrderService {

    private static final Logger log = LoggerFactory.getLogger(CloudTeacherOrderServiceImpl.class);
    @Autowired
    private CloudTeacherOrderDao cloudTeacherOrderDao;
    @Autowired
    private CloudTeacherDao cloudTeacherDao;
    @Autowired
    private StudentService studentService;
    @Autowired
    private MusicGroupPaymentCalenderMemberService musicGroupPaymentCalenderMemberService;
    @Autowired
    private TenantConfigService tenantConfigService;
    @Autowired
    private PayService payService;
    @Autowired
    private IdGeneratorService idGenerator;
    @Autowired
    private TenantOrderRecordService tenantOrderRecordService;

    @Override
    public BaseDAO<Long, CloudTeacherOrder> getDAO() {
        return cloudTeacherOrderDao;
    }

    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private TenantInfoService tenantInfoService;

    @Override
    public boolean addStudents(CloudTeacherAddQueryInfo cloudTeacherAddQueryInfo) {
        for (Integer stId : cloudTeacherAddQueryInfo.getStudentIds()) {
            CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
            cloudTeacherOrder.setStudentId(stId);
            cloudTeacherOrder.setTime(cloudTeacherAddQueryInfo.getTimes());
            cloudTeacherOrder.setType(cloudTeacherAddQueryInfo.getAddType());
            cloudTeacherOrder.setLevel(1);
            cloudTeacherOrder.setMusicGroupId(cloudTeacherAddQueryInfo.getMusicGroupId());
            cloudTeacherOrder.setTenantId(cloudTeacherAddQueryInfo.getTenantId());
            cloudTeacherOrderDao.insert(cloudTeacherOrder);
            this.addStudentCloudTeacher(cloudTeacherOrder);
        }
        return true;
    }

    @Override
    public PageInfo<CloudTeacherStudent> queryRecord(CloudTeacherOrderQueryInfo queryInfo) {
        PageInfo<CloudTeacherStudent> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<CloudTeacherStudent> dataList = null;
        int count = cloudTeacherOrderDao.findRecordCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = cloudTeacherOrderDao.queryRecord(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);

        return pageInfo;
    }

    @Override
    public PageInfo<CloudTeacherStudent> queryInactive(CloudTeacherOrderQueryInfo queryInfo) throws Exception {
        PageInfo<CloudTeacherStudent> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<CloudTeacherStudent> dataList = null;
        int count = cloudTeacherOrderDao.findInactiveCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = cloudTeacherOrderDao.queryInactive(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);

        if (count == 0) {
            return pageInfo;
        }
        assert dataList != null;
        for (CloudTeacherStudent cst : dataList) {
            TenantConfig tenantConfig = tenantConfigService.getOne(new QueryWrapper<TenantConfig>().eq("tenant_id_", cst.getTenantId()));
            if (tenantConfig == null) {
                throw new Exception("未找到组织Id " + cst.getTenantId().toString() + "在 TenantConfig 表");
            }
            JSONObject cfg = JSON.parseObject(tenantConfig.getConfig());
            JSONObject memberConfig = (JSONObject) cfg.get("member_config");
            Double divide;
            switch (cst.getType()) {
                case DAY:
                    divide = memberConfig.getDouble("day_divide");
                    break;
                case MONTH:
                    divide = memberConfig.getDouble("month_divide");
                    break;
                case QUARTERLY:
                    divide = memberConfig.getDouble("quarter_divide");
                    break;
                case YEAR_HALF:
                    divide = memberConfig.getDouble("half_year_divide");
                    break;
                case YEAR:
                    divide = memberConfig.getDouble("year_divide");
                    break;
                default:
                    divide = null;
            }
            if (divide == null) {
                cst.setAmount(null);
            } else {
                cst.setAmount(new BigDecimal(divide * cst.getTime()));
            }
        }
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map<String, Object> pay(List<CloudTeacherStudent> cloudTeacherStudents) throws Exception {

        if (cloudTeacherStudents.size() == 0) {
            throw new Exception("请选择选择一条记录");
        }
        List<Integer> ids = cloudTeacherStudents.stream().map(CloudTeacherStudent::getCloudTeacherOrderId).collect(
                Collectors.toList());
        if (cloudTeacherOrderDao.checkOrdered(StringUtil.join(ids, ",")) != 0) {
            throw new Exception("存在已经支付完成的学生，请重新选择");
        }
        Integer tenantId = null;
        BigDecimal sumAmount = new BigDecimal(0);
        for (CloudTeacherStudent cts : cloudTeacherStudents) {
            if (cts.getAmount() == null) {
                throw new Exception("支付金额异常, 请按规定操作 ");
            }
            sumAmount = sumAmount.add(cts.getAmount());
            if (tenantId == null) {
                tenantId = cts.getTenantId();
            }
        }
        return this.payExec(sumAmount, tenantId, cloudTeacherStudents);
    }

    /**
     * 团练宝支付
     *
     * @param payAmount 金额
     * @param tenantId  分部
     * @return 支付结果
     */
    private Map<String, Object> payExec(BigDecimal payAmount, Integer tenantId, List<CloudTeacherStudent> cloudTeacherStudents) throws Exception {
        //生成订单编号
        String orderNo = idGenerator.generatorId("payment") + "";
        TenantOrderRecordEnum tenantEnum = TenantOrderRecordEnum.CLOUD_TEACHER;

        Map<String, Object> payMap = new HashMap<>();
        int orderState = 0;
        //消费大于0元则拉起支付
        if (payAmount.compareTo(BigDecimal.ZERO) > 0) {
            payMap = payService.getPayToPlatformMap(
                    PaymentChannelEnum.ADAPAY,
                    payAmount,
                    BigDecimal.ZERO,
                    orderNo,
                    null,
                    null,
                    tenantEnum.getMsg(),
                    tenantEnum.getMsg()
            );
        } else { // 消费大于 0 元 已支付 直接激活
            orderState = 1;
        }
        payMap.put("amount", payAmount);
        payMap.put("orderNo", orderNo);
        TenantOrderRecord record = tenantInfoService.createOrderRecord(tenantId, payAmount, orderNo, tenantEnum, orderState, WrapperUtil.toStr(payMap, "routingMerNos"));
        for (CloudTeacherStudent cts : cloudTeacherStudents) {
            if (cloudTeacherOrderDao.updateOrderId(cts.getCloudTeacherOrderId(), cts.getAmount(), record.getId()) <= 0) {
                throw new Exception("更新订单失败");
            }
        }
        //0元直接成功
        if (orderState == 1) {
            tenantOrderRecordService.activeCloudTeacher(record);
        }
        log.info("tenant pay >>>>> " + JSON.toJSONString(payMap));
        return payMap;
    }

    @Override
    public int sendSms(TenantOrderRecord record) {
        List<Mapper> orderUsers = cloudTeacherOrderDao.findUserNameByOrderId(record.getId());
        Map<Integer, String> userPhones = new HashMap<>();
        for (Mapper mapper : orderUsers) {
            userPhones.put((Integer) mapper.getKey(), mapper.getValue().toString());
        }
        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.CLOUD_TEACHER_ORDER_SUCCESS, userPhones, null, 0, null, null);
        return 0;
    }

    public Boolean payCheck(String orderNo) {
        TenantOrderRecord record = tenantOrderRecordService.getOne(new QueryWrapper<TenantOrderRecord>().eq("order_no_", orderNo));
        return record.getOrderState() == 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addOrderDetail2CloudTeacher(StudentPaymentOrder order, BigDecimal cloudTeacherFee) {
        CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
        cloudTeacherOrder.setTenantId(order.getTenantId());
        cloudTeacherOrder.setOrganId(order.getOrganId());
        cloudTeacherOrder.setOrderId(order.getId());
        MusicGroupPaymentCalenderMember calenderMember = musicGroupPaymentCalenderMemberService.findByCalenderId(order.getCalenderId());
        cloudTeacherOrder.setLevel(calenderMember.getMemberRankSettingId());
        cloudTeacherOrder.setTime(calenderMember.getNum().intValue());
        cloudTeacherOrder.setType(calenderMember.getPeriod());
        cloudTeacherOrder.setAmount(cloudTeacherFee);
        cloudTeacherOrder.setStudentId(order.getUserId());
        cloudTeacherOrder.setStatus(1);
        cloudTeacherOrderDao.insert(cloudTeacherOrder);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public CloudTeacher addStudentCloudTeacher(CloudTeacherOrder cloudTeacherOrder) {
        Date nowDate = new Date();
        CloudTeacher cloudTeacher = cloudTeacherDao.getByStudentId(cloudTeacherOrder.getStudentId());
        Date startTime;
        Date endTime = null;
        Date endBaseTime;

        //更新student表信息
        studentService.updateMemberRank(cloudTeacherOrder);

        if (cloudTeacher == null) {
            cloudTeacher = new CloudTeacher();
            startTime = DateUtil.trunc(nowDate);
            endBaseTime = startTime;
            endTime = calcCloudTeacherOrderTime(cloudTeacherOrder, startTime, endTime);
            cloudTeacher.setStudentId(cloudTeacherOrder.getStudentId());
            cloudTeacher.setLevel(cloudTeacherOrder.getLevel());
            cloudTeacher.setStartTime(startTime);
            cloudTeacher.setEndTime(endTime);
            cloudTeacherDao.insert(cloudTeacher);
        } else {
            startTime = cloudTeacher.getStartTime();
            endBaseTime = cloudTeacher.getEndTime();
            if (cloudTeacher.getEndTime().compareTo(nowDate) < 0) {
                startTime = DateUtil.trunc(nowDate);
                endBaseTime = startTime;
            }
            endTime = calcCloudTeacherOrderTime(cloudTeacherOrder, endBaseTime, endTime);
            cloudTeacher.setStartTime(startTime);
            cloudTeacher.setEndTime(endTime);
            cloudTeacher.setUpdateTime(nowDate);
            if (cloudTeacherDao.update(cloudTeacher) <= 0) {
                throw new BizException("团练宝有效期更新失败");
            }
        }
        cloudTeacherOrder.setStartTime(endBaseTime);
        cloudTeacherOrder.setEndTime(endTime);
        cloudTeacherOrder.setStatus(1);
        if (cloudTeacherOrderDao.update(cloudTeacherOrder) <= 0) {
            throw new BizException("团练宝生效处理失败");
        }
        return cloudTeacher;
    }

    public Date calcCloudTeacherOrderTime(CloudTeacherOrder cloudTeacherOrder, Date startTime, Date endTime) {
        if (cloudTeacherOrder.getType() == DAY) { // 日
            endTime = DateUtil.addDays(startTime, cloudTeacherOrder.getTime());
        } else if (cloudTeacherOrder.getType() == MONTH) { // 月
            endTime = DateUtil.addMonths(startTime, cloudTeacherOrder.getTime());
        } else if (cloudTeacherOrder.getType() == QUARTERLY) { // 季
            endTime = DateUtil.addMonths(startTime, cloudTeacherOrder.getTime() * 3);
        } else if (cloudTeacherOrder.getType() == YEAR_HALF) { // 半年
            endTime = DateUtil.addMonths(startTime, cloudTeacherOrder.getTime() * 6);
        } else if (cloudTeacherOrder.getType() == YEAR) { // 年
            endTime = DateUtil.addYears(startTime, cloudTeacherOrder.getTime());
        }
        return endTime;
    }

    @Override
    public List<CloudTeacherOrder> getStudentCloudTeacherOrders(List<Integer> studentIds, String musicGroupId) {
        return cloudTeacherOrderDao.getStudentCloudTeacherOrders(studentIds, musicGroupId);
    }

    @Override
    public List<CloudTeacherOrder> getStudentCloudTeacherOrder(Integer studentId, String musicGroupId) {
        return cloudTeacherOrderDao.getStudentCloudTeacherOrder(studentId, musicGroupId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean quitCloudTeacherOrders(List<CloudTeacherOrder> cloudTeacherOrders, BigDecimal refundAmount) {
        Date nowDate = new Date();
        for (int i = 0; i < cloudTeacherOrders.size(); i++) {
            CloudTeacherOrder cloudTeacherOrder = cloudTeacherOrders.get(i);
            if (cloudTeacherOrder.getStatus().equals(3)) {
                throw new BizException("团练宝已退，请勿重复退");
            }
            if (cloudTeacherOrder.getStatus().equals(2) && DateUtil.stringToDate(DateUtil.format(cloudTeacherOrder.getEndTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT), DateUtil.ISO_EXPANDED_DATE_FORMAT).compareTo(
                    DateUtil.stringToDate(DateUtil.format(nowDate, DateUtil.ISO_EXPANDED_DATE_FORMAT), DateUtil.ISO_EXPANDED_DATE_FORMAT)) < 0) {
                throw new BizException("团练宝已过服务期，不可退费");
            }
            Integer oldStatus = cloudTeacherOrder.getStatus();
            cloudTeacherOrder.setStatus(3);
            if (i == 0) {
                cloudTeacherOrder.setRefundAmount(refundAmount);
            } else {
                cloudTeacherOrder.setRefundAmount(BigDecimal.ZERO);
            }
            cloudTeacherOrder.setUpdateTime(nowDate);
            if (cloudTeacherOrderDao.update(cloudTeacherOrder) <= 0) {
                throw new BizException("团练宝退费处理失败，请重试");
            }

            if (oldStatus.equals(1)) {
                continue;
            }
            int days = DateUtil.daysBetween(DateUtil.trunc(nowDate), cloudTeacherOrder.getEndTime());
            CloudTeacher cloudTeacher = cloudTeacherDao.getByStudentId(cloudTeacherOrder.getStudentId());
            if (cloudTeacher != null) {
                Date endDate = DateUtil.addDays(cloudTeacherOrder.getEndTime(), -days);
                cloudTeacher.setEndTime(endDate);
                cloudTeacher.setUpdateTime(nowDate);
                if (cloudTeacherDao.update(cloudTeacher) <= 0) {
                    throw new BizException("团练宝服务有效期更新失败，请重试");
                }
            }
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean quitCloudTeacherOrder(CloudTeacherOrder cloudOrder, BigDecimal refundAmount) {
        Date nowDate = new Date();
        if (cloudOrder.getStatus().equals(3)) {
            throw new BizException("团练宝已退，请勿重复退");
        }
        if (cloudOrder.getStatus().equals(2) && DateUtil.stringToDate(DateUtil.format(cloudOrder.getEndTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT), DateUtil.ISO_EXPANDED_DATE_FORMAT).compareTo(
                DateUtil.stringToDate(DateUtil.format(nowDate, DateUtil.ISO_EXPANDED_DATE_FORMAT), DateUtil.ISO_EXPANDED_DATE_FORMAT)) < 0) {
            throw new BizException("团练宝已过服务期，不可退费");
        }
        Integer oldStatus = cloudOrder.getStatus();
        cloudOrder.setStatus(3);
        cloudOrder.setRefundAmount(refundAmount);
        cloudOrder.setUpdateTime(nowDate);
        if (cloudTeacherOrderDao.update(cloudOrder) <= 0) {
            throw new BizException("团练宝退费处理失败，请重试");
        }

        if (oldStatus.equals(1)) {
            return true;
        }
        int days = DateUtil.daysBetween(DateUtil.trunc(nowDate), cloudOrder.getEndTime());
        CloudTeacher cloudTeacher = cloudTeacherDao.getByStudentId(cloudOrder.getStudentId());
        Date endDate = DateUtil.addDays(cloudOrder.getEndTime(), -days);
        cloudTeacher.setEndTime(endDate);
        cloudTeacher.setUpdateTime(nowDate);
        if (cloudTeacherDao.update(cloudTeacher) <= 0) {
            throw new BizException("团练宝服务有效期更新失败，请重试");
        }
        return true;
    }

    @Override
    public CloudTeacherOrder queryByOrderId(Long orderId) {
        return cloudTeacherOrderDao.queryByOrderId(orderId);
    }

    @Override
    public CloudTeacherOrderDto queryOrderInfoByOrderId(Long orderId) {
        return cloudTeacherOrderDao.queryOrderInfoByOrderId(orderId);
    }

    @Override
    public List<Map<Long, String>> queryNoStartByUserIds(Set<Integer> userIds, Boolean isMusicMember) {
        return cloudTeacherOrderDao.queryNoStartByUserIds(userIds, isMusicMember);
    }

    @Override
    public String queryActiveOrderPage(Integer userId, Integer activeRemark, String remark) {
        return cloudTeacherOrderDao.queryActiveOrderPage(userId, activeRemark, remark);
    }
}
