package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.ClassGroupDao;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.entity.TenantInfo;
import com.ym.mec.biz.dal.enums.FivePlusGradeEnum;
import com.ym.mec.biz.dal.enums.GradeTypeEnum;
import com.ym.mec.biz.dal.enums.SixPlusGradeEnum;
import com.ym.mec.biz.dal.page.EducationBaseQueryInfo;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.SysEmployeePositionService;
import com.ym.mec.biz.service.TenantInfoService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.SysUserRoleEnum.EDUCATION;

@Service
public class OrganizationServiceImpl extends BaseServiceImpl<Integer, Organization> implements OrganizationService {

	@Autowired
	private OrganizationDao organizationDao;
	@Autowired
	private TenantInfoService tenantInfoService;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private EmployeeDao employeeDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private SysEmployeePositionService employeePositionService;

	@Override
	public BaseDAO<Integer, Organization> getDAO() {
		return organizationDao;
	}

	@Override
	public List<Organization> queryEmployeeOrgan(){
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if(sysUser == null){
			throw new BizException("获取用户信息失败");
		}
		String organIds = this.getEmployeeOrgan(sysUser.getId(), "", sysUser.getIsSuperAdmin());
		return organizationDao.queryEmployeeOrgan(organIds);
	}

	@Override
	public List<Organization> queryEmployeeOrganByUser(){
		return this.queryEmployeeOrgan();
	}

	@Override
	public String getOrganName(String organId) {
		return organizationDao.getOrganName(organId);
	}


	@Override
	public Map<Integer, String> getGradeList(Integer id) {
		Organization organization = organizationDao.get(id);
		Map<Integer, String> gradeMap = new HashMap<>();

		if(Objects.isNull(organization)){
			gradeMap.put(GradeTypeEnum.SIX_PLUS.getCode(), GradeTypeEnum.SIX_PLUS.getDesc());
			return gradeMap;
		}

		if(organization.getGradeType().equals(GradeTypeEnum.FIVE_PLUS)){
			FivePlusGradeEnum[] fivePlusGradeEnums = FivePlusGradeEnum.values();
			for (FivePlusGradeEnum fivePlusGradeEnum : fivePlusGradeEnums) {
				gradeMap.put(fivePlusGradeEnum.getCode(),fivePlusGradeEnum.getDesc());
			}
		}
		if(organization.getGradeType().equals(GradeTypeEnum.SIX_PLUS)){
			SixPlusGradeEnum[] sixPlusGradeEnums = SixPlusGradeEnum.values();
			for (SixPlusGradeEnum sixPlusGradeEnum : sixPlusGradeEnums) {
				gradeMap.put(sixPlusGradeEnum.getCode(),sixPlusGradeEnum.getDesc());
			}
		}
		return gradeMap;
	}

	@Override
	public String getEmployeeOrgan(Integer userId, String organIds,Boolean isSuper) {
		if(StringUtils.isEmpty(organIds)){
			Integer tenantId = TenantContextHolder.getTenantId();
			TenantInfo tenantInfo = tenantInfoService.get(tenantId);
			//如果是超管，或者是机构管理员，可以查看当前机构所有分部数据
			if(isSuper || (tenantInfo.getUserId() != null && tenantInfo.getUserId().equals(userId))){
				Map<String,Object> param = new HashMap<>();
				param.put("tenantId",tenantId);
				List<Organization> all = this.findAll(param);
				if(all != null && all.size() > 0){
					organIds = StringUtils.join(all.stream().map(e->e.getId()).collect(Collectors.toSet()),",");
				}
			}
		}
		if(StringUtils.isEmpty(organIds)){
			Employee employee = employeeDao.get(userId);
			organIds = employee.getOrganIdList();
		}
		return organIds;
	}

	@Override
	public String getEmployeeOrgan(Object organIds) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败");
		}
		return this.getEmployeeOrgan(sysUser.getId(),organIds == null?"":organIds.toString(),sysUser.getIsSuperAdmin());
	}

	@Override
	public String getTeacherOrgan() {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败");
		}
		Teacher teacher = teacherDao.get(sysUser.getId());
		String organIds = teacher.getTeacherOrganId() + "";
		if(StringUtils.isNotBlank(teacher.getFlowOrganRange())){
			organIds = organIds + "," + teacher.getFlowOrganRange();
		}
		return organIds;
	}

	@Override
	public List<Organization> findAllOrgans(Integer tenantId) {
		return organizationDao.findAllOrgans(tenantId);
	}

	@Override
	public List<Organization> findOrgans(List<Integer> organIds,Integer tenantId) {
		return organizationDao.getOrgans(organIds,tenantId);
	}


    @Override
    public <T extends EducationBaseQueryInfo> T onlyEducation(T queryInfo) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败");
		}
		queryInfo.setOrganIds(this.getEmployeeOrgan(sysUser.getId(),queryInfo.getOrganIds(),sysUser.getIsSuperAdmin()));
		Boolean onlyForRole = employeePositionService.onlyForRole(EDUCATION, sysUser.getId(), sysUser.getIsSuperAdmin());
		//如果当前用户只有教务老师角色，那么只能看到他所管理的课程组的信息
		if (onlyForRole) {
			queryInfo.setEducationUserId(sysUser.getId());
		}
		return queryInfo;
    }
}
