package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ExportDto;
import com.ym.mec.biz.dal.dto.QuestionnaireUserResultDto;
import com.ym.mec.biz.dal.dto.ReplacementInstrumentActivityDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ExportEnum;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.QuestionnaireActiveTypeEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.QuestionnaireUserResultQueryInfo;
import com.ym.mec.biz.dal.page.ReplacementInstrumentActivityQueryInfo;
import com.ym.mec.biz.service.CooperationOrganService;
import com.ym.mec.biz.service.ExportService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import com.ym.mec.util.http.HttpUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.biz.service.QuestionnaireUserResultService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.QuestionnaireActiveTypeEnum.CLOUD_TEACHER_FEEDBACK;

@Service
public class QuestionnaireUserResultServiceImpl extends BaseServiceImpl<Long, QuestionnaireUserResult>  implements QuestionnaireUserResultService {
	
	@Autowired
	private QuestionnaireUserResultDao questionnaireUserResultDao;
	@Autowired
	private QuestionnaireQuestionItemDao questionnaireQuestionItemDao;
	@Autowired
	private QuestionnaireTopicDao questionnaireTopicDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private StudentDao studentDao;
	@Autowired
	private QuestionnaireQuestionDao questionnaireQuestionDao;
	@Autowired
	private ExportService exportService;

	@Override
	public BaseDAO<Long, QuestionnaireUserResult> getDAO() {
		return questionnaireUserResultDao;
	}

	@Override
	public void batchInsert(List<QuestionnaireUserResult> questionnaireUserResultList) {
		questionnaireUserResultDao.batchInsert1(questionnaireUserResultList);
	}

	@Override
	public Object queryDetailPage(QueryInfo queryInfo) {
		PageInfo<QuestionnaireUserResultDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<QuestionnaireUserResultDto> dataList = null;
		int count = questionnaireUserResultDao.findDetailCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = questionnaireUserResultDao.queryDetailPage(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public Map<String,Object> questionnaireUserResultExport(String activeType,Integer cooperationId,List<QuestionnaireQuestion> questionList) {
		HashMap<String, Object> paramMap = new HashMap<>(2);
		paramMap.put("activeId",cooperationId);
		paramMap.put("activeType",activeType);
		List<QuestionnaireUserResult> userResults = questionnaireUserResultDao.findAll(paramMap);

		Map<String,Object> resultMap = new HashMap<>(2);
		//导出列表数据
		List<Map<String,Object>> jsonObjectList = new ArrayList<>();
		Set<String> body = new LinkedHashSet<>();

		if(userResults != null && userResults.size() > 0){
			//获取所有的问题列表
			List<Long> questionIdList = questionList.stream().map(e -> e.getId()).collect(Collectors.toList());
			Map<Long, List<QuestionnaireQuestion>> questionMap = questionList.stream().collect(Collectors.groupingBy(QuestionnaireQuestion::getId));
			//获取所有答案列表
			paramMap.put("questionIdList",questionIdList);
			List<QuestionnaireQuestionItem> questionItemList = questionnaireQuestionItemDao.findAll(paramMap);
			Map<Long, List<QuestionnaireQuestionItem>> questionItemMap = questionItemList.stream().collect(Collectors.groupingBy(QuestionnaireQuestionItem::getId));
			//按学员分组
			Set<Integer> userIdSet = new LinkedHashSet<>();
			for (QuestionnaireUserResult userResult : userResults) {
				userIdSet.add(userResult.getUserId());
			}
			List<Integer> userIdList = new ArrayList<>(userIdSet);
			Map<Integer, List<QuestionnaireUserResult>> userIdMap = userResults.parallelStream().collect(Collectors.groupingBy(QuestionnaireUserResult::getUserId));
//			List<Integer> userIdList = new ArrayList<>(userIdMap.keySet());
//			Collections.sort(userIdList);
			Map<Integer, String> nameList;
			if(cooperationId != null){
				nameList = MapUtil.convertMybatisMap(questionnaireQuestionItemDao.queryUserName(cooperationId));
			}else {
				nameList = MapUtil.convertMybatisMap(questionnaireUserResultDao.queryUserName(activeType));
			}
			Map<Integer, String> phoneList = MapUtil.convertMybatisMap(teacherDao.queryPhoneByIds(StringUtils.join(userIdList,",")));

			Map<Integer, String> organNameMap = MapUtil.convertMybatisMap(studentDao.queryStudentOrganNameMap(userIdList));
			for (Integer userId : userIdList) {
				List<QuestionnaireUserResult> userResultList = userIdMap.get(userId);
				if(userResultList != null && userResultList.size() > 0){
					QuestionnaireUserResult result1 = userResultList.get(0);
					Map<String,Object> jsonObject = new LinkedHashMap<>();
					body.add("cooperationId");
					body.add("organName");
					body.add("createTime");
					body.add("userId");
					body.add("userName");
					body.add("phone");
					jsonObject.put("cooperationId",cooperationId);
					jsonObject.put("organName",organNameMap.get(userId));
					jsonObject.put("createTime",result1.getCreateTime());
					jsonObject.put("userId",userId);
					jsonObject.put("userName",nameList.get(userId));
					jsonObject.put("phone",phoneList.get(userId));
					//分解当前学员答案
					for (QuestionnaireUserResult result : userResultList) {
						//获取问题列表
						QuestionnaireQuestion question = questionMap.get(result.getQuestionnaireQuestionId()).get(0);
						String questionnaireQuestionItemIdList = result.getQuestionnaireQuestionItemIdList();
						StringBuffer userResult = new StringBuffer();
						if(StringUtils.isNotEmpty(questionnaireQuestionItemIdList)){
							String[] split = questionnaireQuestionItemIdList.split(",");
							for (String s : split) {
								List<QuestionnaireQuestionItem> items = questionItemMap.get(Long.parseLong(s));
								userResult.append(items.get(0).getAnswerValue()).append("\n");
							}
						}
						String additionalValue = result.getAdditionalValue();
						if(StringUtils.isNotEmpty(additionalValue)){
							userResult.append(additionalValue);
						}
						body.add(question.getContent());
						jsonObject.put(question.getContent(),userResult.toString());
					}
					jsonObjectList.add(jsonObject);
				}
			}
		}
		resultMap.put("jsonObjectList",jsonObjectList);
		resultMap.put("body",body);
		return resultMap;
	}

    @Override
	@Transactional(rollbackFor = Exception.class)
    public void add(ReplacementInstrumentActivityDto replacementInstrumentActivity) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败，请重新登陆");
		}
		Integer topicId = replacementInstrumentActivity.getTopicId();

		List<QuestionnaireUserResult> questionnaireUserResultList = replacementInstrumentActivity.getQuestionnaireUserResultList();
		if(questionnaireUserResultList == null || questionnaireUserResultList.size() == 0){
			throw new BizException("请填写问卷结果");
		}
		QuestionnaireActiveTypeEnum activeType = replacementInstrumentActivity.getActiveType();
		QuestionnaireTopic topic = questionnaireTopicDao.get(topicId);
		if(topic.getAgainCommitFlag() == 0){
			HashMap<String, Object> paramMap = new HashMap<>(3);
			paramMap.put("activeId",replacementInstrumentActivity.getReplacementInstrumentCooperationId());
			paramMap.put("activeType",activeType.getCode());
			paramMap.put("userId",sysUser.getId());
			List<QuestionnaireUserResult> userResults = questionnaireUserResultDao.findAll(paramMap);
			if(userResults != null && userResults.size() > 0){
				throw new BizException("您已提交问卷，请勿重复操作");
			}
		}
		//删除原来的问卷结果
		questionnaireUserResultDao.delByActiveIdAndUserId(null, sysUser.getId(),activeType.getCode());
		for (QuestionnaireUserResult result : questionnaireUserResultList) {
			result.setUserId(sysUser.getId());
			result.setQuestionnaireTopicId(topicId);
			result.setActiveType(activeType);
		}
		//新增问卷结果
		questionnaireUserResultDao.batchInsert1(questionnaireUserResultList);
	}

    @Override
	@Transactional(rollbackFor = Exception.class)
    public void addResult(ReplacementInstrumentActivityDto replacementInstrumentActivity) {
		Integer topicId = replacementInstrumentActivity.getTopicId();

		List<QuestionnaireUserResult> questionnaireUserResultList = replacementInstrumentActivity.getQuestionnaireUserResultList();
		if(questionnaireUserResultList == null || questionnaireUserResultList.size() == 0){
			throw new BizException("请填写问卷结果");
		}
		Long time = new Date().getTime();
		for (QuestionnaireUserResult result : questionnaireUserResultList) {
			result.setUserId(time.intValue());
			result.setQuestionnaireTopicId(topicId);
			result.setActiveType(QuestionnaireActiveTypeEnum.ENTERPRISE_CUSTOMIZATION);
		}
		//新增问卷结果
		questionnaireUserResultDao.batchInsert1(questionnaireUserResultList);
    }

	@Override
	public Object queryResultPage(QuestionnaireUserResultQueryInfo queryInfo) {
		PageInfo<QuestionnaireUserResultDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<QuestionnaireUserResultDto> dataList = null;
		int count = questionnaireUserResultDao.countResultCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = questionnaireUserResultDao.findResultCount(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

    @Override
    public List<String> getExportFields(Integer cooperationId,QuestionnaireActiveTypeEnum activeType,List<QuestionnaireQuestion> questionList) {
		List<String> column = new ArrayList<>();
		if(questionList != null && questionList.size() > 0) {
			column.add("活动编号");
			column.add("分部");
			column.add("提交时间");
			column.add("学员编号");
			column.add("学员姓名");
			column.add("手机号");
			List<String> collect = questionList.stream().map(e -> e.getContent()).collect(Collectors.toList());
			column.addAll(collect);
		}
		return column;
    }

	@Override
	public Set<String> getExportBody(Integer cooperationId, QuestionnaireActiveTypeEnum activeType,List<QuestionnaireQuestion> questionList) {
		if(questionList != null && questionList.size() > 0) {
			Map<String,Object> resultMap = this.questionnaireUserResultExport(activeType.getCode(),cooperationId,questionList);
			return (Set<String>)resultMap.get("body");
		}else {
			throw new BizException("暂无可导出数据");
		}
	}

	@Override
	public List<QuestionnaireQuestion> getQuestionnaireQuestions(Integer cooperationId, QuestionnaireActiveTypeEnum activeType){
		List<QuestionnaireQuestion> questionList;
		if(cooperationId != null){
			questionList = questionnaireQuestionDao.queryQuestionList(cooperationId);
		}else {
			if(activeType != CLOUD_TEACHER_FEEDBACK){
				throw new BizException("报表中心暂不支持 {} 导出",CLOUD_TEACHER_FEEDBACK.getDesc());
			}
			questionList = questionnaireQuestionDao.findByActiveType(activeType.getCode(), TenantContextHolder.getTenantId());
		}
		questionList.removeAll(Collections.singleton(null));
		return questionList;
	}

	@Override
	public void questionnaireUserResultExport(HttpServletResponse response, ExportDto exportDto) throws Exception {
		Map<String, Object> info = exportDto.getQueryInfo();
		Integer cooperationId = exportService.getParam(info, "cooperationId", Integer.class);
		QuestionnaireActiveTypeEnum activeType = QuestionnaireActiveTypeEnum.valueOf(exportService.getParam(info, "activeType", String.class));
		List<QuestionnaireQuestion> questionList = getQuestionnaireQuestions(cooperationId,activeType);
		List<String> exportFields = getExportFields(cooperationId, activeType,questionList);
		List<String> exportBody = new ArrayList<>(getExportBody(cooperationId, activeType,questionList));
		if(exportBody.size() == 0){
			throw new BizException("暂无可导出数据");
		}
		Map<String,String> headMap = new LinkedHashMap<>(exportDto.getHeadColumns().size());
		for (int i = 0; i < exportFields.size(); i++) {
			headMap.put(exportFields.get(i),exportBody.get(i));
		}
		headMap.keySet().removeAll(org.apache.commons.collections.CollectionUtils.subtract(exportFields, exportDto.getHeadColumns()));
		String[] header = headMap.keySet().toArray(new String[headMap.keySet().size()]);
		String[] body = headMap.values().toArray(new String[headMap.keySet().size()]);
		Map<String,Object> resultMap = this.questionnaireUserResultExport(activeType.getCode(),cooperationId,questionList);
		List<Map<String,Object>> jsonObjectList = (List<Map<String,Object>>)resultMap.get("jsonObjectList");
		OutputStream outputStream = response.getOutputStream();
		HSSFWorkbook workbook = null;
		try {
			workbook = POIUtil.exportExcel(header, body, jsonObjectList);
			response.setContentType("application/octet-stream");
			response.setHeader("Content-Disposition", "attachment;filename=replacement-" + DateUtil.getDate(new Date()) + ".xls");
			response.flushBuffer();
			outputStream = response.getOutputStream();
			workbook.write(outputStream);
			outputStream.flush();
			workbook.close();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (outputStream != null) {
				try {
					workbook.close();
					outputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
}