package com.ym.service.Impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.http.HttpHelper;
import com.ym.mec.biz.dal.dao.ImLiveBroadcastRoomDao;
import com.ym.mec.biz.dal.entity.ImLiveBroadcastRoom;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.im.IMHelper;
import com.ym.pojo.IMApiResultInfo;
import com.ym.pojo.RecordConfig;
import com.ym.pojo.RecordNotify;
import com.ym.service.LiveRoomService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.HttpURLConnection;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hgw
 * Created by 2022-02-21
 */
@Slf4j
@Service
public class LiveRoomServiceImpl implements LiveRoomService {

    @Autowired
    private IMHelper imHelper;
    @Autowired
    private HttpHelper httpHelper;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private ImLiveBroadcastRoomDao imLiveBroadcastRoomDao;

    /**
     * 创建房间-聊天室
     *
     * @param roomId   房间Uid
     * @param roomName 房间名称
     */
    public IMApiResultInfo createLiveRoom(String roomId, String roomName) {
        IMApiResultInfo resultInfo;
        try {
            resultInfo = imHelper.createChatRoom(roomId, roomName);
        } catch (Exception e) {
            log.error("create chatRoom error >>>", e.getCause());
            throw new BizException("创建聊天室失败!");
        }
        if (!resultInfo.isSuccess()) {
            log.error("create chatRoom error: {}", resultInfo.getErrorMessage());
            throw new BizException("创建聊天室失败!");
        }
        log.info("create chatRoom success: {}", roomId);
        return resultInfo;
    }

    /**
     * 销毁房间-聊天室
     *
     * @param roomId 房间Uid
     */
    public IMApiResultInfo destroyLiveRoom(String roomId) {
        //删除服务器房间
        List<String> deleteRoomIds = new ArrayList<String>() {{
            add(roomId);
        }};
        IMApiResultInfo resultInfo;
        try {
            resultInfo = imHelper.deleteChrm(deleteRoomIds);
        } catch (Exception e) {
            throw new BizException("关闭聊天室失败!");
        }
        if (!resultInfo.isSuccess()) {
            log.error("destroy chatRoom error: {}", resultInfo.getErrorMessage());
            throw new BizException("关闭聊天室失败!");
        }
        return resultInfo;
    }

    @Override
    public void startRecord(String roomId) throws Exception {
        JSONObject paramJson = new JSONObject();
        paramJson.put("sessionId",getRoomSessionId(roomId));
        ImLiveBroadcastRoom imLiveBroadcastRoom = imLiveBroadcastRoomDao.selectById(roomId);
        paramJson.put("config",new RecordConfig(imLiveBroadcastRoom.getSpeakerId()));
        String body = paramJson.toJSONString();
        HttpURLConnection conn = httpHelper.createIMRtcPostHttpConnection("/rtc/record/start.json", "application/json",roomId);
        httpHelper.setBodyParameter(body, conn);
        IMApiResultInfo resultInfo = JSON.parseObject(httpHelper.returnResult(conn, body), IMApiResultInfo.class);
        if(resultInfo.getCode() != 200){
            log.error("直播视频录制失败：resultInfo : {} ",resultInfo);
        }
    }

    @Override
    public void recordSync(RecordNotify recordNotify) {
        log.info("recordSync paramJson{}",recordNotify);
    }

    public String getRoomSessionId(String roomId){
        RBucket<String> bucket = redissonClient.getBucket("sessionId:" + roomId);
        String sessionId = bucket.get();
        if(StringUtils.isNotEmpty(sessionId)){
            return sessionId;
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("roomId",roomId);

        HttpURLConnection conn = null;
        try {
            conn = httpHelper.createIMRtcPostHttpConnection("/rtc/room/query.json", "application/json",null);
            httpHelper.setBodyParameter(jsonObject.toJSONString(), conn);
            String returnResult = httpHelper.returnResult(conn, jsonObject.toJSONString());
            JSONObject resultObject = JSONObject.parseObject(returnResult);
            String code = resultObject.get("code").toString();
            if("200".equals(code)){
                bucket.set(resultObject.get("sessionId").toString());
            }else {
                log.error("获取sessionId失败 returnResult:{}",returnResult);
                throw new BizException("获取sessionId失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return sessionId;
    }

}
