package com.ym.mec.web.controller;


import com.ym.mec.biz.dal.dto.ImLiveBroadcastRoomDto;
import com.ym.mec.biz.dal.vo.ImLiveBroadcastRoomVo;
import com.ym.mec.biz.service.ImLiveBroadcastRoomService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.WrapperUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Map;

/**
 * 直播房间管理表(ImLiveBroadcastRoom)表控制层
 *
 * @author hgw
 * @since 2022-02-17 20:52:04
 */
@Api(tags = "直播房间管理表")
@RestController
@RequestMapping("/imLiveBroadcastRoom")
public class ImLiveBroadcastRoomController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private ImLiveBroadcastRoomService imLiveBroadcastRoomService;

    @ApiImplicitParams({
            @ApiImplicitParam(name = "search", dataType = "String", value = "模糊查询关键字"),
            @ApiImplicitParam(name = "liveState", dataType = "Integer", value = "直播状态 0未开始 1开始 2结束"),
            @ApiImplicitParam(name = "startTime", dataType = "String", value = "开始时间"),
            @ApiImplicitParam(name = "endTime", dataType = "String", value = "结束时间"),
            @ApiImplicitParam(name = "page", dataType = "Integer", value = "页数"),
            @ApiImplicitParam(name = "rows", dataType = "Integer", value = "每页数量"),
    })
    @ApiOperation("分页查询直播间列表")
    @PostMapping("/queryPage")
    public HttpResponseResult<PageInfo<ImLiveBroadcastRoomVo>> queryPage(@RequestBody Map<String, Object> param) {
        return succeed(imLiveBroadcastRoomService.queryPage(param));
    }

    @ApiOperation("查询房间信息")
    @GetMapping("/queryRoom")
    public HttpResponseResult<ImLiveBroadcastRoomVo> queryRoom(@ApiParam(value = "房间uid", required = true) String roomUid,
                                                               @ApiParam(value = "用户id", required = true) Integer userId) {
        return succeed(imLiveBroadcastRoomService.queryRoom(roomUid, userId));
    }

    @ApiOperation("查询房间信息")
    @GetMapping("/queryRoomInfo")
    public HttpResponseResult<ImLiveBroadcastRoomVo> queryRoomInfo(@ApiParam(value = "房间uid", required = true) String roomUid) {
        return succeed(imLiveBroadcastRoomService.queryRoomInfo(roomUid));
    }

    @ApiOperation("创建直播间")
    @PostMapping("/add")
//    @PreAuthorize("@pcs.hasPermissions('imLiveBroadcastRoom/add')")
    public HttpResponseResult add(@Valid @RequestBody ImLiveBroadcastRoomDto dto) {
        imLiveBroadcastRoomService.add(dto);
        return succeed();
    }

    @ApiOperation("修改直播间信息-已开播无法修改")
    @PostMapping("/update")
//    @PreAuthorize("@pcs.hasPermissions('imLiveBroadcastRoom/update')")
    public HttpResponseResult update(@Valid @RequestBody ImLiveBroadcastRoomDto dto) {
        imLiveBroadcastRoomService.update(dto);
        return succeed();
    }

    /**
     * 关闭直播间
     *
     * @param id 房间表id
     */
    @ApiOperation("关闭直播间")
    @GetMapping(value = "/roomDestroy/{id}")
//    @PreAuthorize("@pcs.hasPermissions('imLiveBroadcastRoom/roomDestroy')")
    public HttpResponseResult roomDestroy(@ApiParam(value = "房间表id", required = true) @PathVariable("id") Integer id) {
        imLiveBroadcastRoomService.roomDestroy(id);
        return succeed();
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", dataType = "Integer", value = "房间id"),
    })
    @ApiOperation("删除直播间信息-已开播无法删除")
    @PostMapping("/delete")
//    @PreAuthorize("@pcs.hasPermissions('imLiveBroadcastRoom/delete')")
    public HttpResponseResult delete(@RequestBody Map<String, Object> param) {
        Integer id = WrapperUtil.toInt(param, "id", "请传入房间id");
        imLiveBroadcastRoomService.delete(id);
        return succeed();
    }

    @ApiOperation("同步点赞数量")
    @GetMapping("/syncLike")
    public HttpResponseResult syncLike(@ApiParam(value = "房间uid", required = true) String roomUid,
                                       @ApiParam(value = "点赞数", required = true) Integer likeNum) {
        imLiveBroadcastRoomService.syncLike(roomUid, likeNum);
        return succeed();
    }

    @ApiOperation("进入房间")
    @GetMapping("/joinRoom")
    public void joinRoom(String roomUid, Integer userId) {
        imLiveBroadcastRoomService.joinRoom(roomUid, userId);
    }

    @GetMapping("/test")
    public HttpResponseResult test() {
        imLiveBroadcastRoomService.test();
        return succeed();
    }

    @GetMapping("/shareGroup")
    public HttpResponseResult shareGroup(@ApiParam(value = "房间uid", required = true) String roomUid,
                                         @ApiParam(value = "群编号", required = true) String groupIds) {
        imLiveBroadcastRoomService.shareGroup(roomUid,groupIds);
        return succeed();
    }

}

