package com.ym.mec.web.controller;


import com.ym.mec.biz.dal.dto.TenantOrderRecordDto;
import com.ym.mec.biz.service.OrderPayOpsService;
import com.ym.mec.biz.service.TenantOrderRecordService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Map;

/**
 * 机构付款记录表(TenantOrderRecord)表控制层
 *
 * @author hgw
 * @since 2021-12-20 10:08:27
 */
@RestController
@Api(tags = "机构付款记录")
@RequestMapping("tenantOrderRecord")
public class TenantOrderRecordController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private TenantOrderRecordService tenantOrderRecordService;
    @Autowired
    private OrderPayOpsService orderPayOpsService;

    @ApiOperation("分页查询")
    @PostMapping(value = "/queryPage")
    public Object queryPage(@RequestBody TenantOrderRecordDto dto) {
        return tenantOrderRecordService.queryPage(dto);
    }

    @ApiOperation(value = "自测用的")
    @PostMapping("/executePayment")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "amount", value = "支付金额", required = true, dataType = "BigDecimal"),
            @ApiImplicitParam(name = "orderNo", value = "订单号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "payChannel", value = "支付方式（alipay-支付宝app支付）", required = true, dataType = "String"),
    })
    public Object executePayment(BigDecimal amount, String orderNo, String payChannel, String notifyUrl, String returnUrl, String orderSubject, String orderBody, String sign, String code, String platform) {
        try {
            Map<String, Object> payment = orderPayOpsService.executePayment(amount, orderNo, payChannel, returnUrl, orderSubject, orderBody, sign, code, platform);
            return succeed(payment);
        } catch (Exception e) {
            return failed(e.getMessage());
        }
    }

    @ApiOperation(value = "自测用的")
    @PostMapping("/checkTenantOrder")
    public Object checkTenantOrder(String orderNo) {
        return succeed(tenantOrderRecordService.checkTenantOrder(orderNo));
    }

}

